"use strict";
/*global angular, window, cordova, app, Image, document, ionic */

angular.module('$authService', []).service('$authService', ['$rootScope', '$http', '$iot', function ($rootScope, $http, $iot) {

    var state = this;

    //state.apiDevice = "https://2mk786bvj2.execute-api.us-east-1.amazonaws.com/v2stage";
    state.apiDevice = "https://3zera33ryl.execute-api.us-west-2.amazonaws.com/v2prod";
    //state.providerName = 'login.tti.stage';
    state.providerName = 'login.tti.prod';
    //state.poolId = 'us-east-1:f67b4493-2802-4f47-91e1-dcebc748e6e9';
    state.poolId = 'us-west-2:abc0bf83-dd65-4212-9494-981956e07292';
    state.uid = uuid.v1();
    state.key = null;
    state.username = null;
    state.password = null;
    state.sessionInfo = null;
    state.currentClientState = null;
    state.isConnect = false;

    state.clearAll = function () {
        state.key = null;
        state.username = null;
        state.password = null;
        state.sessionInfo = null;
        state.currentClientState = null;
        state.isConnect = false; 
    }

    state.getApiGatewayClient = function (credentials, cbGetApiClient) {
        state.sessionInfo = JSON.parse(window.localStorage.getItem("sessionInfo"))
        if (credentials !== false) {
            state.username = credentials.username
            state.password = CryptoJS.MD5(credentials.password).toString();
            checkAWSCredentials(function(err){
                if (err) {
                    state.key = null;
                    return cbGetApiClient(err, null);
                }
                var config = {
                    accessKey: AWS.config.credentials.accessKeyId,
                    secretKey: AWS.config.credentials.secretAccessKey,
                    sessionToken: AWS.config.credentials.sessionToken,
                    invokeUrl: state.apiDevice,
                    region: state.poolId.split(':')[0]
                };
                window.localStorage.setItem("sessionInfo", JSON.stringify(config));
                var apiClient = apigClientFactory.newClient(config);
                var deviceToken = window.localStorage.getItem('token');
                var deviceUUID = (window.device && window.device.uuid) ?  window.device.uuid : "browserTestUUID";
                if (deviceToken) {

                    //CONFIG IOT cc79cf9408bf
                    //$iot.start("robot/cc79cfae078c/response/get_map/11223344", config.region, AWS.config.credentials)

                    var tokenExist = function (onDone) {
                        var userId = window.localStorage.getItem('userId');
                        apiClient.Get('/foundry/devices/by-type', {type_id: "ce0ab930-3eed-11e7-9b5d-5763b2661e96", account_id: userId}).then(function (result) {
                            var deviceExist = false;
                            result.data.forEach(function (d) {
                                if (d.deviceId === deviceUUID) deviceExist = true;
                            })
                            if (typeof onDone === "function") onDone(deviceExist);
                        })
                    };
                    var setDevice = function (token, uid, exist) {
                        var body = {
                            "a.push_notification_token": token,
                            "accountId": window.localStorage.getItem('userId'),
                            "deviceId": uid
                        }
                        if (exist) {
                            apiClient.Put('/foundry/device', {}, body).then(function (result) {})
                        } else {
                            body.typeId = 'ce0ab930-3eed-11e7-9b5d-5763b2661e96';
                            apiClient.Post('/foundry/device', {}, body).then(function (result) {})
                        }
                    };
                    tokenExist(function (exist) {
                        setDevice(deviceToken, deviceUUID, exist);
                    });


                }
                return cbGetApiClient(null,apiClient);
            })
        }
        if (credentials === false && state.sessionInfo && state.sessionInfo !== null) {
            var apiClient = apigClientFactory.newClient(state.sessionInfo);
            return cbGetApiClient(null,apiClient); 
        }
    }

    var login = function (username, password, cbLogin) {
        var timestamp = new Date().toISOString();
        var signature = CryptoJS.HmacSHA256(timestamp, computeDecryptionKey(username, password));
        var hexSign = signature.toString(CryptoJS.enc.Hex);
        var loginURL = state.apiDevice + '/foundry/login?uid='+ state.uid + '&username=' + encodeURIComponent(username) + '&timestamp='+ timestamp +'&signature=' + hexSign;

        $http({method: 'GET', url: loginURL}).then(function successCallback(response) {
            console.log("success-login->", response);
            if (response.status !== 200) return cbLogin("error");

            var key1 = computeDecryptionKey(username, password).substring(0,32);
            var json = decrypt(response.data, key1);

            if (json && json.user && json.user.robart_user_login) {
                window.localStorage.setItem("robart_user_login", json.user.robart_user_login)
            }

            state.key = json.key;
            window.localStorage.setItem('userId', json.user.account_id);

            getToken(state.uid, username, null, function(err,data){
                cbLogin(err);
            });

          }, function errorCallback(response) {
            console.log("error-login->", response);
            var standardErr = "Incorrect username or password.";
            var errorMSg = (response.data.error === "Invalid signature") ? standardErr : (response.data.error || standardErr)
            return cbLogin(errorMSg);
          });
    };

    function computeDecryptionKey(username, password) {
        var l = document.createElement("a");
        l.href = state.apiDevice;
        var host = l.hostname;
        var salt = username +  state.providerName + host;
        var hashedSalt = CryptoJS.HmacSHA256(salt, password);
        return hashedSalt.toString(CryptoJS.enc.Hex);
    };

    function decrypt(data,key){
        if(typeof data == 'string') {
            var byteCharacters = window.atob(data);
            var iv = new Array(16);
            var crypttext = new Array(byteCharacters.length - 16);
            for (var i = 0; i < byteCharacters.length; i++) {
                if (i < 16)
                    iv[i] = byteCharacters.charCodeAt(i);
                else
                    crypttext[i - 16] = byteCharacters.charCodeAt(i);
            }
            var cipherParams = CryptoJS.lib.CipherParams.create({
                ciphertext: byteArrayToWordArray(crypttext)
            });
            var plaintextArray = CryptoJS.AES.decrypt(
                cipherParams,
                CryptoJS.enc.Hex.parse(key),
                {iv: byteArrayToWordArray(iv), mode: CryptoJS.mode.CBC, keySize: 128}
            );
            return JSON.parse(CryptoJS.enc.Latin1.stringify(plaintextArray));
        } else {
            return data;
        }
    };

    function byteArrayToWordArray(ba) {
        var wa = [], i;
        for (i = 0; i < ba.length; i++) {
            wa[(i / 4) | 0] |= ba[i] << (24 - 8 * i);
        }
        return CryptoJS.lib.WordArray.create(wa, ba.length);
    }

    function checkAWSCredentials(cbCheckAWSCredentials) {
        var currentTime = moment().valueOf();
        var expireTime = 0;
        if (AWS.config.credentials !== null) {
            expireTime = moment(AWS.config.credentials.expireTime).valueOf();
        }
        if ((AWS.config.credentials === null || AWS.config.credentials.expired || currentTime >= expireTime)) {
            if (state.uid && state.key) {
                getToken(state.uid, state.username, null, function(err){
                    cbCheckAWSCredentials(err);
                });
            } else {
                login(state.username, state.password, function(err,data){
                    cbCheckAWSCredentials(err);
                })
            }
        } else {
            cbCheckAWSCredentials(null);
        }
    };

    function getToken(uid, username, identityId, cbGetToken) {
        var logins = {};
        logins[state.providerName] = username;
        var timestamp = new Date().toISOString();
        var stringToSign = timestamp;
        var providerParams = '';
        var loginCount = 1;
        for (var provider in logins){
            if (logins.hasOwnProperty(provider)) {
                stringToSign += provider + logins[provider];
                providerParams += '&provider'+loginCount+'='+provider+'&token'+loginCount+'='+encodeURIComponent(logins[provider]);
                loginCount++;
            }
        }
        if (identityId) {
            stringToSign += identityId;
            providerParams += '&identityId='+encodeURIComponent(identityId);
        }
        var signature = CryptoJS.HmacSHA256(stringToSign, state.key);
        var hexSign = signature.toString(CryptoJS.enc.Hex);
        var getTokenURL = state.apiDevice + '/foundry/token?uid='+ uid + '&timestamp='+ timestamp + providerParams + '&signature=' + hexSign;
        $http({method: 'GET', url: getTokenURL}).then(function successCallback(response) {
            console.log("success-token->", response);
            if (response.status !== 200) return cbGetToken("error");
            var json = decrypt(response.data, state.key);

            getAWSCredentials(json.identityId,json.token, function(err){
                cbGetToken(err);
            })
          }, function errorCallback(response) {
            console.log("error-token->", response);
            return cbGetToken("error");
          });
    };

    function getAWSCredentials(identity_id, token, cbGetAWSCredentials) {
        var region = state.poolId.split(':')[0];
        AWS.config.region = region;
        AWS.config.credentials = new AWS.CognitoIdentityCredentials({
            IdentityPoolId: state.poolId,
            IdentityId: identity_id,
            Logins: {
                'cognito-identity.amazonaws.com': token
            }
        });
        AWS.config.credentials.get(function (err) {
            if (err) {
                return cbGetAWSCredentials(err);
            }
            return cbGetAWSCredentials(null);
        });
    };

}]);
