"use strict";
/*global angular, window, cordova, app, Image, document, ionic */

angular.module('$iot', []).service('$iot', ['$rootScope', '$http', function ($rootScope, $http) {

    var state = this;

    //state.poolId = 'us-east-1:f67b4493-2802-4f47-91e1-dcebc748e6e9';
    state.poolId = 's-west-2:abc0bf83-dd65-4212-9494-981956e07292';
    state.clientId = "7LTevL9m9Cp8ipGFLtVQk";
    state.iotClient = null;

    function SigV4Utils() {}

    SigV4Utils.getSignatureKey = function (key, date, region, service) {
        var kDate = AWS.util.crypto.hmac('AWS4' + key, date, 'buffer');
        var kRegion = AWS.util.crypto.hmac(kDate, region, 'buffer');
        var kService = AWS.util.crypto.hmac(kRegion, service, 'buffer');
        var kCredentials = AWS.util.crypto.hmac(kService, 'aws4_request', 'buffer');    
        return kCredentials;
    };

    SigV4Utils.getSignedUrl = function(host, region, credentials) {
        var datetime = AWS.util.date.iso8601(new Date()).replace(/[:\-]|\.\d{3}/g, '');
        var date = datetime.substr(0, 8);

        var method = 'GET';
        var protocol = 'wss';
        var uri = '/mqtt';
        var service = 'iotdevicegateway';
        var algorithm = 'AWS4-HMAC-SHA256';

        var credentialScope = date + '/' + region + '/' + service + '/' + 'aws4_request';
        var canonicalQuerystring = 'X-Amz-Algorithm=' + algorithm;
        canonicalQuerystring += '&X-Amz-Credential=' + encodeURIComponent(credentials.accessKeyId + '/' + credentialScope);
        canonicalQuerystring += '&X-Amz-Date=' + datetime;
        canonicalQuerystring += '&X-Amz-SignedHeaders=host';

        var canonicalHeaders = 'host:' + host + '\n';
        var payloadHash = AWS.util.crypto.sha256('', 'hex')
        var canonicalRequest = method + '\n' + uri + '\n' + canonicalQuerystring + '\n' + canonicalHeaders + '\nhost\n' + payloadHash;

        var stringToSign = algorithm + '\n' + datetime + '\n' + credentialScope + '\n' + AWS.util.crypto.sha256(canonicalRequest, 'hex');
        var signingKey = SigV4Utils.getSignatureKey(credentials.secretAccessKey, date, region, service);
        var signature = AWS.util.crypto.hmac(signingKey, stringToSign, 'hex');

        canonicalQuerystring += '&X-Amz-Signature=' + signature;
        if (credentials.sessionToken) {
            canonicalQuerystring += '&X-Amz-Security-Token=' + encodeURIComponent(credentials.sessionToken);
        }

        var requestUrl = protocol + '://' + host + uri + '?' + canonicalQuerystring;
        return requestUrl;
    };

    state.start = function (topic, region, credentials) {

        console.log(region);
        console.log(credentials);

        var dataUrl = 'data.iot.' + region + '.amazonaws.com';
        var requestUrl = SigV4Utils.getSignedUrl(dataUrl, region, credentials)

        state.iotClient = new Paho.MQTT.Client(requestUrl, state.clientId);

        var connectOptions = {
            onSuccess: function() {
                console.log("SUCCESS");
                state.iotClient.subscribe(topic)
            },
            useSSL: true,
            timeout: 30,
            keepAliveInterval: 0,
            mqttVersion: 4,
            onFailure: function(e) {
                console.log("ERROR!!!!!!!!!");
                console.log(e);
            }
        };
        state.iotClient.connect(connectOptions);

        state.iotClient.onMessageArrived = function(message) {
            console.log("MESSAGE---");
            var raw = message.payloadBytes;



            var buffer = raw.buffer;

            console.log(buffer);
            console.log("INFLATING");

            try {
                console.log(111111);
                var result = pako.inflate(buffer);
                console.log(result);
            } catch (e) {
                console.log("EEEEEEEE");
                console.log(e);
            }
            

            //console.log(result);


            /*try {
                if (message && message.destinationName && message.payloadString && !isNaN(message.qos)) {
                    //handleIotMessage(message.destinationName, message.payloadString, message.qos);
                }
            } catch (e) {
                console.log("error! Thing Shadow " + e);
            }*/
        };

        state.iotClient.onConnectionLost = function(responseObj) {
                console.log( 'Lost connection');
                console.log(responseObj);
                if (!responseObj || responseObj.errorCode ==0 ) return;
                setTimeout(function () {
                    //awsSocketReconnect();
                }, 5000); //to have time to remove the incorrect subscription
        }


    };

    state.send = function (message, topic) {
        if (state.iotClient !== null) {
            console.log("SENDING...");
            var message = new Paho.MQTT.Message(message);
            message.destinationName = topic;
            state.iotClient.send(message);
        }
    };

}]);
